@@ -11,8 +11,8 @@
  *
  * GAWK is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 1, or (at your option)
- * any later version.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * GAWK is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,7 +21,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with GAWK; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 /*
@@ -69,10 +69,18 @@
  *	file-spec is left in the argument list rather than having it expand
  *	into thin  air.   No  attempt is made to identify  and	make $(var)
  *	environment substitutions--must draw the line somewhere!
+ *
+ *   Oct'91, gawk 2.13.3
+ *	Open '<' with full sharing allowed, so that we can  read batch logs
+ *	and other open files.  Create record-format output ('>$') with read
+ *	sharing permited,  so that others can read our output file to check
+ *	progess.  For stream  output ('>' or  '>>'), sharing is  disallowed
+ *	(for performance reasons).
  */
 
 #include "awk.h"	/* really "../awk.h" */
 #include "vms.h"
+#include <lnmdef.h>
 
        void   v_add_arg(int, const char *);
 static char  *skipblanks(const char *);
@@ -90,14 +98,14 @@ void
 vms_arg_fixup( int *pargc, char ***pargv )
 {
     char *f_in, *f_out, *f_err,
-	*out_mode, *rms_opt1, *rms_opt2;
+	*out_mode, *rms_opt1, *rms_opt2, *rms_opt3, *rms_opt4;
     char **argv = *pargv;
     int i, argc = *pargc;
     int err_to_out_redirect = 0, out_to_err_redirect = 0;
 
-#ifndef NO_CHECK_SHELL
+#ifdef CHECK_DECSHELL	    /* don't define this if linking with DECC$SHR */
     if (shell$is_shell())
-	return;		    /* don't do anything if we're running DECshell */
+	return;		    /* don't do anything if we're running DEC/Shell */
 #endif
 #ifndef NO_DCL_CMD
     for (i = 1; i < argc ; i++)     /* check for dash or other non-VMS args */
@@ -108,11 +116,13 @@ vms_arg_fixup( int *pargc, char ***pargv )
 	v_argz = v_argc = 0,  v_argv = NULL;
     }
 #endif
-    v_add_arg(v_argc = 0, basename(argv[0]));	/* store arg #0 (image name) */
+    v_add_arg(v_argc = 0, argv[0]);	/* store arg #0 (image name) */
 
     f_in = f_out = f_err = NULL;	/* stdio setup (no filenames yet) */
     out_mode = "w";			/* default access for stdout */
     rms_opt1 = rms_opt2 = "ctx=stm";	/* ("context = stream") == no-opt */
+    rms_opt3 = "shr=nil";		/* no sharing (for '>' output file) */
+    rms_opt4 = "mrs=0";			/* maximum record size */
 
     for (i = 1; i < argc; i++) {
 	char *p, *fn;
@@ -146,9 +156,10 @@ vms_arg_fixup( int *pargc, char ***pargv )
 		else if (*p == '&')	/* '>&' => stderr */
 		    is_out = 0,  p++;
 		else if (*p == '$')	/* '>$' => kludge for record format */
-		    rms_opt1 = "rfm=var",  rms_opt2 = "rat=cr",  p++;
+		    rms_opt1 = "rfm=var",  rms_opt2 = "rat=cr",
+		    rms_opt3 = "shr=get",  rms_opt4 = "mrs=32767",  p++;
 		else			/* '>'	=> create */
-		    ;	    /* use default values initialized prior to loop */
+		    {}	    /* use default values initialized prior to loop */
 		p = skipblanks(p);
 		fn = (*p ? p : argv[++i]);	/* use next arg if necessary */
 		if (i >= argc || *fn == '-') {
@@ -196,8 +207,9 @@ vms_arg_fixup( int *pargc, char ***pargv )
 		if (*(p+1) == '\0' && i == argc - 1) {
 		    fatal("background tasks not available ('&' encountered)");
 		    break;
-		} else		/* fall through */
+		} else {	/* fall through */
 		    ;	/*NOBREAK*/
+		}
 	  case '-':		/* argument */
 		is_arg = 1;		/*(=> skip wildcard check)*/
 	  default:		/* other (filespec assumed) */
@@ -236,39 +248,38 @@ ordinary_arg:
 	 && (f_err[len] == ':' || f_err[len] == '\0'))
 	    err_to_out_redirect = 1;
 	else
-	    vms_define("SYS$ERROR", f_err);
+	    (void) vms_define("SYS$ERROR", f_err);
     }
-    /* do stdin before stdout, so we bomb we won't create empty output file */
+    /* do stdin before stdout, so if we bomb we won't make empty output file */
     if (f_in) {		/* [re]open file and define logical name */
-	stdin = freopen(f_in, "r", stdin, "mbf=2");
+	stdin = freopen(f_in, "r", stdin,
+			"ctx=rec", "shr=get,put,del,upd",
+			"mrs=32767", "mbc=24", "mbf=2");
 	if (stdin != NULL)
-	    vms_define("SYS$INPUT", f_in);
+	    (void) vms_define("SYS$INPUT", f_in);
 	else
 	    fatal("<%s (%s)", f_in, strerror(errno));
     }
-    if (f_out) {	/* disallow file sharing to reduce overhead */
+    if (f_out) {
 	stdout = freopen(f_out, out_mode, stdout,
-			 rms_opt1, rms_opt2, "shr=nil", "mbf=2");   /*VAXCRTL*/
-	if (stdout != NULL) {
-#ifdef crtl_bug  /* eof sometimes doesn't get set properly for stm_lf file */
-# define BIGBUF 8*BUFSIZ	/* maximum record size: 4096 instead of 512 */
-	    setvbuf(stdout, malloc(BIGBUF), _IOFBF, BIGBUF);
-#endif
-	    vms_define("SYS$OUTPUT", f_out);
-	} else
+			 rms_opt1, rms_opt2, rms_opt3, rms_opt4,
+			 "mbc=24", "mbf=2");
+	if (stdout != NULL)
+	    (void) vms_define("SYS$OUTPUT", f_out);
+	else
 	    fatal(">%s%s (%s)", (*out_mode == 'a' ? ">" : ""),
 		  f_out, strerror(errno));
     }
     if (err_to_out_redirect) {	/* special case for ``2>&1'' construct */
-	fclose(stderr);
-	dup(1, 2);	/* make file 2 (stderr) share file 1 (stdout) */
+	(void) fclose(stderr);
+	(void) dup2(1, 2);	/* make file 2 (stderr) share file 1 (stdout) */
 	stderr = stdout;
-	vms_define("SYS$ERROR", "SYS$OUTPUT:");
+	(void) vms_define("SYS$ERROR", "SYS$OUTPUT:");
     } else if (out_to_err_redirect) {	/* ``1>&2'' */
-	fclose(stdout);
-	dup(2, 1);	/* make file 1 (stdout) share file 2 (stderr) */
+	(void) fclose(stdout);
+	(void) dup2(2, 1);	/* make file 1 (stdout) share file 2 (stderr) */
 	stdout = stderr;
-	vms_define("SYS$OUTPUT", "SYS$ERROR:");
+	(void) vms_define("SYS$OUTPUT", "SYS$ERROR:");
     }
 
 #ifndef NO_DCL_CMD
@@ -341,8 +352,6 @@ v_add_arg( int idx, const char *val )
 	    fatal("%s: %s: can't allocate memory (%s)", "vms_args",
 		  "v_argv", strerror(errno));
 	} else {
-	    memmsg((oldsize == 0 ? "v_argv" : "re: v_argv"), v_argz,
-		   "vms_args", v_argv);
 	    while (old_size < v_argz)  v_argv[old_size++] = NULL;
 	}
     }
@@ -366,18 +375,21 @@ skipblanks( const char *ptr )
 static u_long
 vms_define( const char *log_name, const char *trans_val )
 {
-    Dsc log_dsc, trn_dsc;
-# define LOG_PROCESS_TABLE 2		/* <obsolete> */
-# define LOG_USERMODE 3			/* PSL$C_USER */
-    extern u_long SYS$CRELOG();		/* <superceded by $CRELNM> */
+    Dsc log_dsc;
+    static Descrip(lnmtable,"LNM$PROCESS_TABLE");
+    static long attr = LNM$M_CONFINE;
+    static Itm itemlist[] = { {sizeof attr,LNM$_ATTRIBUTES,&attr,0},
+			      {0,LNM$_STRING,0,0}, {0,0} };
+    static unsigned char acmode = PSL$C_USER;
 
     /* avoid "define SYS$OUTPUT sys$output:" for redundant ">sys$output:" */
     if (strncasecmp(log_name, trans_val, strlen(log_name)) == 0)
 	return 0;
 
     log_dsc.len = strlen(log_dsc.adr = (char *)log_name);
-    trn_dsc.len = strlen(trn_dsc.adr = (char *)trans_val);
-    return SYS$CRELOG(LOG_PROCESS_TABLE, &log_dsc, &trn_dsc, LOG_USERMODE);
+    itemlist[1].buffer = (char *)trans_val;
+    itemlist[1].len = strlen(trans_val);
+    return SYS$CRELNM((u_long *)0, &lnmtable, &log_dsc, &acmode, itemlist);
 }
 
 /* t_strstr -- strstr() substitute; search 'str' for 'sub' */
