@@ -10,8 +10,8 @@
  *
  * GAWK is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 1, or (at your option)
- * any later version.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * GAWK is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with GAWK; see the file COPYING.  If not, write to
- * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 #include "awk.h"	/* really "../awk.h" */
@@ -42,8 +42,8 @@ static int find_c( const char *s, int n, char c ) {
 #define is_stdout(file_no) ((file_no) == 1)	/* fileno(stdout) */
 #define is_stderr(file_no) ((file_no) == 2)	/* fileno(stderr) */
 
-#define PREFIX_CR  (0x8D << 16) /* leading carriage return */
-#define POSTFIX_CR (0x8D << 24) /* trailing carriage return (=> lf/cr) */
+#define PREFIX_CR  0x008D0000	/* leading carriage return */
+#define POSTFIX_CR 0x8D000000	/* trailing carriage return (=> lf/cr) */
 
 static short  channel[_NFILE] = {0};
 static FILE  *prev_file = 0;
@@ -121,7 +121,7 @@ tty_fwrite( const void *buf, size_t size, size_t number, FILE *file )
 		if (vmsfail(sts))  break;
 	    }
 	    /* queue an asynchronous write */
-	    sts = SYS$QIO(evfn, chan, io_func, &iosb, (u_long (*)())0, 0,
+	    sts = SYS$QIO(evfn, chan, io_func, &iosb, (void (*)())0, 0L,
 			  pt, pos, 0, cc_fmt, 0, 0);
 	    if (vmsfail(sts))  break;	/*(should never happen)*/
 	    pt += pos,	count -= pos;
